<?php

	include ("functions.php");

	function fifth_insight_activate_plugin(){

		if(version_compare(get_bloginfo("version"), "4.2", "<")){
			wp_die( __("You must update the wordpress before using this plugin", "fifth_insight") );
		}

		global $wpdb;
		$query = "
			CREATE TABLE IF NOT EXISTS `". $wpdb->prefix ."fifth_insight_api_key`  (
				`id` BIGINT (20) PRIMARY KEY AUTO_INCREMENT,
				`created` DATETIME DEFAULT CURRENT_TIMESTAMP,
				`api_key` TEXT
			)
		";

		require (ABSPATH . "/wp-admin/includes/upgrade.php");
		dbDelta($query);

		$query = "
			
			CREATE TABLE IF NOT EXISTS `". $wpdb->prefix ."fifth_insight_report` (
				`id` BIGINT(20) PRIMARY KEY AUTO_INCREMENT,
				`url` VARCHAR(255),
				`created` DATETIME,
				`filename` VARCHAR(255),
				`best-practices` FLOAT, 
				`accessibility` FLOAT,
				`seo` FLOAT,
				`performance` FLOAT,
				`type` TINYINT(1),
				`avg` FLOAT
			)
		";

		dbDelta($query);

		if(!file_exists(WP_CONTENT_DIR . DIRECTORY_SEPARATOR . 'data')){
			mkdir(WP_CONTENT_DIR . DIRECTORY_SEPARATOR . 'data');
		}

	}

	function fifth_insight_deactivate_plugin(){

		global $wpdb;
		$wpdb->query("DROP TABLE IF EXISTS `". $wpdb->prefix."fifth_insight_api_key`");
		$wpdb->query("DROP TABLE IF EXISTS `".$wpdb->prefix."fifth_insight_report`");
	
		fifth_insight_delete_data_dir(WP_CONTENT_DIR . DIRECTORY_SEPARATOR . "data");

	}

	function fifth_insight_admin_init(){
		global $submenu;
		$submenu_list = $submenu["analyze-performance"];
		return $submenu_list;
	}

	function fifth_insight_show_data(){

		global $wpdb;



		if(isset($_GET["page"])){
			if(esc_html(sanitize_text_field($_GET["page"] !== "analyze-performance"))){
				return;
			}
		}


		$error = "";

		if(isset($_GET["error"]) && esc_html(sanitize_text_field($_GET["error"])) === "true"){
			$error = "Please enter api key";
		}

		global $wpdb;

		$apiExists = $wpdb->get_var("SELECT COUNT(*) FROM `". $wpdb->prefix ."fifth_insight_api_key`");
		if($apiExists > 0){
			$result = array_shift($wpdb->get_results("SELECT * FROM `". $wpdb->prefix ."fifth_insight_api_key`"));
			$pages = get_pages();

			include (plugin_dir_path(dirname(__FILE__)) . "front/dashboard.php");
		}else{
			include (plugin_dir_path(dirname(__FILE__)) . "front/setup.php");
		}
	}

	function fifth_insight_report_page(){
		$output = fifth_insight_get_navbar(array("active-nav", "", ""));
		global $wpdb;

		$result = $wpdb->get_results("SELECT accessibility, performance, `avg`, seo, `best-practices` as prac from `". $wpdb->prefix ."fifth_insight_report` order by created desc, type asc limit 2");

		if(sizeof($result) < 2){
            $result = array_shift($wpdb->get_results("SELECT * FROM `". $wpdb->prefix ."fifth_insight_api_key`"));
			include (plugin_dir_path(dirname(__FILE__)) . "front/report_.php");
		}else{
			include (plugin_dir_path(dirname(__FILE__)) . "front/report.php");
		}
	}

	function fifth_insight_comparison_page(){
		$output = fifth_insight_get_navbar(array("", "active-nav", ""));
		global $wpdb;

		if(isset($_GET["id"])){
			$url = get_page_link(esc_html(sanitize_text_field($_GET["id"])));
			$result = $wpdb->get_results("SELECT accessibility, performance, `avg`, seo, `best-practices` as prac from `". $wpdb->prefix ."fifth_insight_report` WHERE url = '". $url ."' order by created desc limit 4");
		}else{

			$rs = $wpdb->get_results("SELECT url FROM `". $wpdb->prefix ."fifth_insight_report` GROUP BY url HAVING COUNT(*) >= 4 LIMIT 1");
			$result = $wpdb->get_results("SELECT accessibility, performance, `avg`, seo, `best-practices` as prac from `". $wpdb->prefix ."fifth_insight_report` WHERE url='". $rs[0]->url ."' order by created desc limit 4");

		}



		if(sizeof($result) < 4){
            $result = array_shift($wpdb->get_results("SELECT * FROM `". $wpdb->prefix ."fifth_insight_api_key`"));
			include (plugin_dir_path(dirname(__FILE__)) . "front/comparison_.php");
		}else{
			include (plugin_dir_path(dirname(__FILE__)) . "front/comparison.php");
		}
	}

	function fifth_insight_full_report(){
		$output = fifth_insight_get_navbar(array("", "", "active-nav"));

		global $wpdb;

		$mobile = "";
		$desktop = "back-primary";

		if(isset($_GET["id"])){

			$url = get_page_link(esc_html(sanitize_text_field($_GET["id"])));
			$latestData = $wpdb->get_results("SELECT filename, created, url from `". $wpdb->prefix ."fifth_insight_report` WHERE url='". $url ."'  order by created limit 1");

		}else{
			$latestData = $wpdb->get_results("SELECT filename, created, url from `". $wpdb->prefix ."fifth_insight_report` order by created limit 1");
		}

		if(sizeof($latestData) == 0){
			include (plugin_dir_path(dirname(__FILE__)) . "front/full_report_.php");
		}else{
			$filename = WP_CONTENT_DIR . DIRECTORY_SEPARATOR . "data" . DIRECTORY_SEPARATOR . $latestData[0]->filename;
			$content = json_decode(file_get_contents($filename));

			if(isset($_GET["strategy"]) && esc_html(sanitize_text_field($_GET["strategy"])) === "mobile"){
				$mobile = "back-primary text-white";
				$desktop = "text-dark";
				$content = fifth_insight_get_full_report($content->data, "mobile");
			}else{
				$content = fifth_insight_get_full_report($content->data, "desktop");
			}

			$best_practises = fifth_insight_get_category_audits($content, "best-practices");
			$accessibility = fifth_insight_get_category_audits($content, "accessibility");
			$seo = fifth_insight_get_category_audits($content, "seo");
			$performance = fifth_insight_get_category_audits($content, "performance");
			//print_r($best_practises);

			$pages = get_pages();

			include (plugin_dir_path(dirname(__FILE__)) . "front/full_report.php");

		}


		
	}

	function fifth_insight_enqueue_scripts(){
		wp_register_style("fifth_insight_bootstrap", plugins_url("/".FIFTHINSIGHT_PLUGIN_URI."/assets/css/bootstrap.min.css"), FIFTHINSIGHT_PLUGIN_URL);
		wp_register_style("fifth_insight_font",  plugins_url("/".FIFTHINSIGHT_PLUGIN_URI."/assets/css/font-awesome.min.css"), FIFTHINSIGHT_PLUGIN_URL);
		wp_register_style("fifth_insight_custom_style", plugins_url("/".FIFTHINSIGHT_PLUGIN_URI."/assets/css/fifthinsight.css"), FIFTHINSIGHT_PLUGIN_URL);
		wp_enqueue_style("fifth_insight_bootstrap");
		wp_enqueue_style("fifth_insight_font");
		wp_enqueue_style("fifth_insight_custom_style");

		wp_register_script("fifth_insight_bootstrap_script", plugins_url("/". FIFTHINSIGHT_PLUGIN_URI . "/assets/js/bootstrap.min.js"), FIFTHINSIGHT_PLUGIN_URL);
		wp_register_script("fifth_insight_popper_script", plugins_url("/". FIFTHINSIGHT_PLUGIN_URI . "/assets/js/popper.min.js"), FIFTHINSIGHT_PLUGIN_URL);
		wp_register_script("fifth_insight_jquery_circle", plugins_url("/". FIFTHINSIGHT_PLUGIN_URI . "/assets/js/jquery.circle.progress.min.js"), FIFTHINSIGHT_PLUGIN_URL);
		wp_register_script("fifth_insight_custom_js", plugins_url("/".FIFTHINSIGHT_PLUGIN_URI."/assets/js/fifthinsight.js"), FIFTHINSIGHT_PLUGIN_URL);

		wp_localize_script("fifth_insight_custom_js", "pages_obj", array(
			"ajax_url"  =>  admin_url() . "admin-ajax.php"
		));
		wp_enqueue_script("jquery");
		wp_enqueue_script("fifth_insight_popper_script");
		wp_enqueue_script("fifth_insight_bootstrap_script");
		wp_enqueue_script("fifth_insight_jquery_circle");
		wp_enqueue_script("fifth_insight_custom_js");
	}

	function fifth_insight_admin_menu(){
		add_menu_page(
			"Performance Test by WP_Catalysts",
			"FifthInsight",
			"manage_options",
			"analyze-performance",
			"fifth_insight_show_data",
			"",
			4
		);

		add_submenu_page(
			"analyze-performance",
			"Overview",
			"Overview",
			"manage_options",
			"performance-report",
			"fifth_insight_report_page"
		);

		add_submenu_page(
			"analyze-performance",
			"Comparison",
			"Comparison",
			"manage_options",
			"performance-comparison",
			"fifth_insight_comparison_page"
		);

		add_submenu_page(
			"analyze-performance",
			"Full Report",
			"Full Report",
			"manage_options",
			"performance-full-report",
			"fifth_insight_full_report"
		);

		add_submenu_page(
			"analyze-performance",
			"Settings",
			"Settings",
			"manage_options",
			"performance-setting",
			"fifth_insight_setting"
		);
	}

	function fifth_insight_api_save(){

		if($_POST["api_key"] === ""){
			wp_redirect(admin_url("admin.php?page=analyze-performance&&error=true"));
		}else{
			global $wpdb;
			check_admin_referer("fifth_insight_api_save");

			$wpdb->insert($wpdb->prefix . "fifth_insight_api_key", array("api_key" => sanitize_text_field($_POST["api_key"])));
			wp_redirect(admin_url("admin.php?page=analyze-performance&&set=true"));
		}

	}

	function fifth_insight_setting(){

		$api_key = "";
		global $wpdb;
		$api = $wpdb->get_results("SELECT api_key from `". $wpdb->prefix ."fifth_insight_api_key`");
		
		$output = "";

		if(sizeof($api) !== 0){
			$api_key = $api[0]->api_key;
		}
		
		if(isset($_GET["error"])){
			$output = fifth_insight_display_alert_msg("Something went wrong while saving API Key", "alert-danger");
		}

		if(isset($_GET["update"])){
			$output = fifth_insight_display_alert_msg("API Key updated successfully", "alert-success");
		}

		if(isset($_GET["feedback"])){

			if(esc_html(sanitize_text_field($_GET["feedback"])) === "true"){
				
				$output = fifth_insight_display_alert_msg("Feedback Sent successfully", "alert-success");

			}else{
				$output = fifth_insight_display_alert_msg("Something went wrong in sending feedback", "alert-success");
			}
			
		}

		if(isset($_GET["contact"])){

			if(esc_html(sanitize_text_field($_GET["contact"])) === "true"){
				
				$output = fifth_insight_display_alert_msg("Developer contacted successfully", "alert-success");

			}else{
				$output = fifth_insight_display_alert_msg("Something went wrong in sending email", "alert-success");
			}
			
		}

		
		
		include (plugin_dir_path(dirname(__FILE__)) . "front/setting.php");
	}

	function fifth_insight_feedback(){
	
		if(fifth_insight_send_mail(sanitize_email($_POST["email"]), esc_html(sanitize_text_field($_POST["body"])), "Feedback related to plugin")){
			wp_redirect(admin_url("admin.php?page=performance-setting&&feedback=true"));
		}else{
			wp_redirect(admin_url("admin.php?page=performance-setting&&feedback=false"));
		}
		
	}

	function fifth_insight_contact(){
		
		if(fifth_insight_send_mail(sanitize_email($_POST["email"]), esc_html(sanitize_text_field($_POST["body"])), "Contact us form")){
			wp_redirect(admin_url("admin.php?page=performance-setting&&contact=true"));
		}else{
			wp_redirect(admin_url("admin.php?page=performance-setting&&contact=false"));
		}

	}

	function fifth_insight_api_update(){

		if(esc_html($_POST["api_key"]) === ""){
			wp_redirect(admin_url("admin.php?page=performance-setting&&error=true"));
		}else{
			global $wpdb;
			$api = $wpdb->get_results("SELECT api_key from `". $wpdb->prefix ."fifth_insight_api_key`");
			check_admin_referer("fifth_insight_api_update");


			if(sizeof($api) == 0){
				
				$wpdb->insert(
					$wpdb->prefix . "fifth_insight_api_key",
					array( 	
						"api_key"	=>	sanitize_text_field($_POST["api_key"])
					)
				);

			}else{
				
				$wpdb->update(
					$wpdb->prefix . "fifth_insight_api_key",
					array("api_key" => sanitize_text_field($_POST["api_key"])),
					array("api_key" => $api[0]->api_key)
				);
			}

			wp_redirect(admin_url("admin.php?page=performance-setting&&update=true"));
		}

	}

/**
 *
 */
function fifth_insight_get_pages(){

		$dateObj = new DateTime('now', new DateTimeZone('Asia/Kolkata'));
		$date = $dateObj->format("d.m.Y.H.i.s");
		$dateNow = $dateObj->format("Y-m-d H:i:s");
		$data = json_encode($_POST);

		global $wpdb;

		$posturl = sanitize_text_field($_POST["data"][0]["url"]);
		$resultCount = $wpdb->get_var("SELECT count(*) from `". $wpdb->prefix ."fifth_insight_report` WHERE url = '".$posturl."'");

		if($resultCount >= 4){
			$existingData = $wpdb->get_results("SELECT filename from `". $wpdb->prefix ."fifth_insight_report` WHERE url = '".$posturl."' order by created limit 1");
			$filename = WP_CONTENT_DIR . DIRECTORY_SEPARATOR . "data" . DIRECTORY_SEPARATOR . $existingData[0]->filename;
			$wpdb->delete($wpdb->prefix . "fifth_insight_report", array("filename" => $existingData[0]->filename));
			unlink($filename);
		}


		$filename = $date . "-report.json";
		$filename = substr($filename, 0, strlen($filename));

		$obj = array();
		$record = array();

		$record["url"] = sanitize_text_field($_POST["data"][0]["url"]);
		$record["created"] = $dateNow;
		$record["filename"] = $filename;

		array_push($obj, $record);
		array_push($obj, $record);


		$sum = array("desktop" => 0, "mobile" => 0);
		foreach($_POST["data"] as $row){
			$sum[$row["strategy"]] += sanitize_text_field( $row["score"] );
			if($row["strategy"] == "desktop"){
				$obj[0][$row["category"]] =  sanitize_text_field( $row["score"] );
			}else{
				$obj[1][$row["category"]] =  sanitize_text_field( $row["score"] );
			}
		}



		$obj[0]["type"] = 0;
		$obj[1]["type"] = 1;

		$obj[0]["avg"] = round($sum["desktop"]/4, 2);
		$obj[1]["avg"] = round($sum["mobile"]/4, 2);


		$wpdb->insert($wpdb->prefix . "fifth_insight_report", $obj[0]);
		$wpdb->insert($wpdb->prefix . "fifth_insight_report", $obj[1]);

		if(file_put_contents(
			WP_CONTENT_DIR . DIRECTORY_SEPARATOR . "data/" . $filename,
			$data,
			FILE_APPEND
		)){
			echo "File written";
		}


	}
